// Avoid `console` errors in browsers that lack a console.
(function() {
    var method;
    var noop = function () {};
    var methods = [
        'assert', 'clear', 'count', 'debug', 'dir', 'dirxml', 'error',
        'exception', 'group', 'groupCollapsed', 'groupEnd', 'info', 'log',
        'markTimeline', 'profile', 'profileEnd', 'table', 'time', 'timeEnd',
        'timeStamp', 'trace', 'warn'
    ];
    var length = methods.length;
    var console = (window.console = window.console || {});

    while (length--) {
        method = methods[length];

        // Only stub undefined methods.
        if (!console[method]) {
            console[method] = noop;
        }
    }
}());

// namespace it
( function( obj ) {

	/**************************************************************/
	
	obj.swfID 			= null;	// swf id
	obj.swf 			= null;	// swf object
	obj.unityParentObj 	= null;	// unity parent object (doing the embed)
	obj.unity 			= null; // unity object
	obj.isFlashReady	= false;// is flash ready to start?
	obj.isUnityReady	= false;// is unity ready to start?
	obj.txtMsgs			= null;	// the messages element
	obj.numMsgs			= 0;	// the number of messages sent

	/**************************************************************/
	
	/**
	 * Inits the Fish object
	 * @param swfID The ID of our SWF file
	 * @param unityParentObj The parent object that's embedding our unity file
	 */
	obj.init = function( swfID, unityParentObj ) {
		obj.swfID			= swfID;
		obj.unityParentObj	= unityParentObj;
	}
	
	/**
	 * Called when flash sends a message to unity or vice versa
	 */
	obj.addMsgSent = function() {
		obj.numMsgs++;
		obj.getTxtMsgs().innerHTML = "Since you've been here, Flash and Unity have sent " + obj.numMsgs + " messages to each other";
	}
	
	/**
	 * Gets the text paragraph object
	 * @return The paragraph object, or null if we can't find it
	 */
	obj.getTxtMsgs = function() {
		
		// if we already have it, just return
		if( obj.txtMsgs != null )
			return obj.txtMsgs;
	
		if( document.getElementById )
			obj.txtMsgs = document.getElementById( "numMsgs" );
		else if( document.getElementByName )
			obj.txtMsgs = document.getElementByName( "numMsgs" );
			
		// error if we couldn't find it
		if( obj.txtMsgs == null )
			console.error( "Can't find the txtMsgs 'numMsgs' as both getElementById() and getElementByName() aren't supported" );
		return obj.txtMsgs;
	}
	
	/**
	 * Gets the swf object in the page
	 * @return The SWF object, or null if we can't find it
	 */
	obj.getSWF = function() {
	
		// if we already have it, just return
		if( obj.swf != null )
			return obj.swf;
	
		if( document.getElementById )
			obj.swf = document.getElementById( obj.swfID );
		else if( document.getElementByName )
			obj.swf = document.getElementByName( obj.swfID );
			
		// error if we couldn't find it
		if( obj.swf == null )
			console.error( "Can't find the swf '" + obj.swfID + "' as both getElementById() and getElementByName() aren't supported" );
		return obj.swf;
	};
	
	/**
	 * Gets the unity object in the page
	 * @return The Unity object, or null if we can't find it
	 */
	obj.getUnity = function() {
	
		// if we already have it, just return
		if( obj.unity != null )
			return obj.unity;
			
		// we need our parent object
		if( obj.unityParentObj == null )
			console.error( "Can't get the unity object as we don't have it's parent" );
		else
		{
			obj.unity = obj.unityParentObj.getUnity();
			if( obj.unity == null )
				console.error( "Can't find the unity object" );
		}
		return obj.unity;
	};	

	/**************************************************************/
	/** FLASH FUNCTIONS */
	
	/**
	 * Called when our flash object is ready
	 */
	obj.FlashReady = function() {
		console.log( "Flash is ready!" );
		obj.isFlashReady = true;		
		obj._checkIfAllReady();
	}
	
	/**
	 * Called when the paused property of the game is changed
	 * @param isPaused Is the pause true, or false
	 */
	obj.OnPauseChange = function( isPaused ) {
		obj._callUnity( "OnPauseFishGame", ( isPaused ) ? 1 : 0 );
	}
	
	/**
	 * Called when the ship has started to move
	 * @param dir The direction that we're moving (-1, 1)
	 */
	obj.OnShipStartMove = function( dir ) {
		obj._callUnity( "OnShipStartMove", dir );
	}
	
	/**
	 * Called when the ship has stopped moving
	 * @param xPC the % for the x position, between 0 and 1
	 */
	obj.OnShipStopMove = function( xPC ) {
		obj._callUnity( "OnShipStopMove", xPC );
	}
	
	/**
	 * Called when the ship has throw out a love tnt barrel
	 * @param xPC The % for the barrel x position, between 0 and 1
	 */
	obj.OnLoveTNT = function( xPC ) {
		obj._callUnity( "OnLoveTNT", xPC );
	}
	
	/**
	 * Called when the ship wants to blow up the tnt
	 */
	obj.OnLoveTNTExplode = function() {
		obj._callUnity( "OnLoveTNTExplode" );
	}
	
	/**
	 * Called when a fish has landed
	 * @param xPC The % for the fish x position, between 0 and 1
	 * @param angle The angle that the fish is entering at
	 * @param type The fish type
	 * @param scale The fish scale
	 * @param speed The fish speed
	 */
	obj.OnFishLand = function( xPC, angle, type, scale, speed ) {
		obj._callUnity( "OnFishLand", xPC + "," + angle + "," + type + "," + scale + "," + speed );
	}
	
	/**
	 * Called when the line has started to move
	 * @param dir The direction that we're moving (-1, 1)
	 */
	/* obj.OnLineStartMove = function( dir ) {
		obj._callUnity( "OnLineStartMove", dir );
	} */
	
	/**
	 * Called when the line has stopped moving
	 * @param y The line's y position, in meters
	 */
	/* obj.OnLineStopMove = function( y ) {
		obj._callUnity( "OnLineStopMove", y );
	} */
	
	/**
	 * Called when the position of our ship has changed
	 * @param xPC The % for the x position, between 0 and 1
	 */
	/* obj.ShipPosChange = function( xPC ) {
		obj._callUnity( "ShipPosChange", xPC );
	} */
	
	/**
	 * Called when the position of our line has changed
	 * @param sx The x position of the start of our line, in meters
	 * @param sy The y position of the start of our line, in meters
	 * @param ex The x position of the end of our line, in meters
	 * @param ey The y position of the end of our line, in meters
	 */
	/* obj.LinePosChange = function( sx, sy, ex, ey ) {
		obj._callUnity( "LinePosChange", sx + "," + sy + "," + ex + "," + ey );
	} */
	
	/**
	 * Called when flash or unity get key input - we have the JS be the master
	 * control, that way we only need to listen for one
	 * @param keyCode The keycode of the key pressed
	*/
	/* obj.OnKeyDown = function( keyCode ) {
		obj.getSWF().OnKeyDown( keyCode );
	} */
	
	/**
	 * Called when flash or unity get key input - we have the JS be the master
	 * control, that way we only need to listen for one
	 * @param keyCode The keycode of the key release
	*/
	/* obj.OnKeyUp = function( keyCode ) {
		obj.getSWF().OnKeyUp( keyCode );
	} */

	/**************************************************************/
	/** UNITY FUNCTIONS */
	
	/**
	 * Called when our unity is ready
	 */
	obj.UnityReady = function() {
		console.log( "Unity is ready!" );
		obj.isUnityReady = true;
		obj._checkIfAllReady();
	}
	
	/**
	 * Called when unity is telling us a fish is jumping
	 * @param params The params of the fish jump, as a String
	 */
	obj.OnFishJump = function( params ) {
		obj.getSWF().OnFishJump( params );
		obj.addMsgSent();
	}

	/**************************************************************/
	
	// checks if both are ready to move on
	obj._checkIfAllReady = function() {
		if( obj.isFlashReady && obj.isUnityReady )
		{
			// set up our window key presses, so we can get input when neither the
			// flash or unity are focused
			/* window.onkeydown = function (e) {
				obj.OnKeyDown( e.keyCode ? e.keyCode : e.which );
			};
			window.onkeyup = function (e) {
				obj.OnKeyUp( e.keyCode ? e.keyCode : e.which );
			} */
			
			// tell flash and unity that we're ready
			obj.getSWF().GlobalReady();
			obj._callUnity( "GlobalReady" );
			obj.addMsgSent(); // _callUnity adds one
		}
	}
	
	// sends a message to unity
	obj._callUnity = function( func, params ) {
		var str = ( params != null ) ? ( ( ( typeof params ) === 'string' || ( typeof params ) === 'number' ) ? params : JSON.stringify( params ) ) : "";
		obj.getUnity().SendMessage( "JSReceiver", func, str );
		obj.addMsgSent();
	}
}( window.Fish = window.Fish || {} ) );	