package fishing
{
	import com.divillysausages.ds.DS;
	import fishing.input.Controls;
	import fishing.network.Ext;
	import fishing.state.constants.StateNames;
	import fishing.state.FSM;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;

	/**
	 * ...
	 * @author Damian Connolly
	 */
	[Frame(factoryClass="fishing.Preloader")]
	public class Main extends Sprite 
	{
		
		/********************************************************************************/
		
		/**
		 * Our game state machine
		 */
		public var fsm:FSM = null;
		
		/**
		 * Our external object (JS), that will notify unity etc
		 */
		public var js:Ext = null;
		
		/**
		 * The controls class
		 */
		public var controls:Controls = null;
		
		/********************************************************************************/
		
		private var m_blocker:Sprite 	= null; // the blocker to stop us clicking on anything
		private var m_btnPause:Sprite	= null;	// the button to unpause us
		
		/********************************************************************************/

		public function Main():void 
		{
			if ( this.stage) this._init();
			else this.addEventListener( Event.ADDED_TO_STAGE, this._init );
		}
		
		/********************************************************************************/

		private function _init( e:Event = null ):void 
		{
			this.removeEventListener(Event.ADDED_TO_STAGE, _init);
			
			// start our engine
			DS.start( this );
			DS.onPauseChange 				= this._onPauseChange;
			DS.updateManager.useFixedTime 	= false;
			
			// create our state machine, and start us off
			this.fsm = new FSM;
			this.fsm.changeState( StateNames.INIT );
		}
		
		// is the paused changed?
		private function _onPauseChange( isPaused:Boolean ):void
		{
			if ( isPaused )
			{
				// create our blocker is needed
				if ( this.m_blocker == null )
				{
					this.m_blocker = new Sprite;
					this.m_blocker.graphics.beginFill( 0xff0000, 0.0 );
					this.m_blocker.graphics.drawRect( 0.0, 0.0, DS.stage.stageWidth, DS.stage.stageHeight );
					this.m_blocker.graphics.endFill();
					this.m_blocker.addEventListener( MouseEvent.CLICK, function( me:MouseEvent ):void
					{
						// do nothing
						me.stopPropagation();
						me.stopImmediatePropagation();
						me.preventDefault();
					});
				}
				DS.stage.addChild( this.m_blocker );
				
				// create our button if needed
				if ( this.m_btnPause == null )
				{
					this.m_btnPause = new Sprite;
					this.m_btnPause.graphics.lineStyle( 2.0, 0xffffff );
					this.m_btnPause.graphics.beginFill( 0x000066 );
					this.m_btnPause.graphics.drawRect( 0.0, 0.0, 100.0, 50.0 );
					this.m_btnPause.graphics.endFill();
					this.m_btnPause.x = DS.stage.stageWidth * 0.5 - this.m_btnPause.width * 0.5;
					this.m_btnPause.y = DS.stage.stageHeight * 0.5 - this.m_btnPause.height * 0.5;
					this.m_btnPause.addEventListener( MouseEvent.CLICK, function( me:MouseEvent ):void
					{
						DS.paused = false;
					});
					
					// text
					var tf:TextFormat 	= new TextFormat( "Trebuchet Ms", 12, 0xffffff, true );
					var t:TextField		= new TextField;
					t.defaultTextFormat	= tf;
					t.autoSize			= TextFieldAutoSize.LEFT;
					t.selectable		= false;
					t.mouseEnabled		= false;
					t.mouseWheelEnabled	= false;
					t.wordWrap			= false;
					t.multiline			= false;
					t.text				= "READY";
					t.x					= this.m_btnPause.width * 0.5 - t.width * 0.5;
					t.y					= this.m_btnPause.height * 0.5 - t.height * 0.5;
					this.m_btnPause.addChild( t );
				}
				DS.stage.addChild( this.m_btnPause );
			}
			else
			{
				// remove our elements
				if ( this.m_blocker.parent != null )
					this.m_blocker.parent.removeChild( this.m_blocker );
				if ( this.m_btnPause.parent != null )
					this.m_btnPause.parent.removeChild( this.m_btnPause );
			}
			
			// notify js
			this.js.notifyPauseChanged( isPaused );
		}

	}

}