package fishing.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import fishing.state.constants.SizeConstants;
	import flash.display.Sprite;
	import org.osflash.signals.Signal;
	
	/**
	 * The barrel that we throw out of the ship
	 * @author Damian Connolly
	 */
	public class Barrel extends Sprite implements IUpdateObj
	{
		
		/********************************************************************************/
		
		/**
		 * The signal dispatched when we start going out of the screen. It should take no
		 * parameters
		 */
		public var signalOnHitStageLimit:Signal = null;
		
		public var hasShownSplash:Boolean = false;
		
		public var origHeight:Number = 0.0;
		
		/********************************************************************************/
		
		private var m_xVel:Number 		= 0.0;
		private var m_yVel:Number 		= 0.0; // the velocity for the y
		private var m_width:Number		= 0.0;	// the width for the barrel at 0 rotation
		private var m_graphics:barrel	= null;
		
		/********************************************************************************/
		
		public function get active():Boolean { return !this.isOutOfScreen; }
		
		public function get isOutOfScreen():Boolean { return this.y > DS.stage.stageHeight + this.height; }
		
		public function get isGoingDown():Boolean { return this.m_yVel > 0.0; }
		
		public function get isGoingRight():Boolean { return this.m_xVel > 0.0; }
		
		/********************************************************************************/
		
		public function Barrel() 
		{
			// add our graphics
			this.m_graphics = new barrel;
			this.m_width	= this.m_graphics.width;
			this.origHeight	= this.m_graphics.height;
			this.addChild( this.m_graphics );
			DS.addToUpdate( this );
			
			this.signalOnHitStageLimit = new Signal;
		}
		
		public function destroy():void
		{
			DS.removeFromUpdate( this );
			
			// signals
			this.signalOnHitStageLimit.removeAll();
			
			// grpahics
			this.removeChildren();
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			// clear our params
			this.signalOnHitStageLimit 	= null;
			this.m_graphics				= null;
		}
		
		/**
		 * Throws the barrel out of the boat
		 * @param xVel The x velocity of the barrel
		 * @param yVel The y velocity of the barrel
		 */
		public function throwOut( xVel:Number, yVel:Number ):void
		{
			this.m_xVel = xVel;
			this.m_yVel = yVel;
		}
		
		public function update( dt:Number ):void
		{
			// update the vel, unless we hit the ocean
			var oceanFloor:Number = DS.stage.stageHeight - SizeConstants.OCEAN_SIZE_PX;
			if ( this.y < oceanFloor )
				this.m_yVel += 9.8 * dt;
			else
				this.m_xVel *= 0.9;
				
			// move along
			var prevY:Number = this.y;
			this.x += this.m_xVel;
			this.y += this.m_yVel;
			
			// clamp the x
			this.x = ( this.x < this.m_width * 0.5 ) ? this.m_width * 0.5 :
						( this.x > DS.stage.stageWidth - this.m_width * 0.5 ) ? DS.stage.stageWidth - this.m_width * 0.5 :
							this.x;
			
			// check if we're just going out of screen
			if ( prevY + this.height * 0.5 < DS.stage.stageHeight && this.y + this.height * 0.5 >= DS.stage.stageHeight )
				this.signalOnHitStageLimit.dispatch();
			
			// rotate, unless we've passed the ocean floor, in which case, straighten up
			if( this.y < oceanFloor )
				this.m_graphics.rotation += this.m_xVel * 4;
			else
				this.m_graphics.rotation = MathHelper.lerp( this.m_graphics.rotation, 0, 0.5 )
			
			// if we're out of frame, then destroy
			if ( this.isOutOfScreen )
				this.destroy();
		}
		
	}

}