package fishing.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import com.divillysausages.ds.util.Colour;
	import fishing.assets.Resources;
	import fishing.state.constants.SizeConstants;
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	import org.osflash.signals.Signal;
	/**
	 * ...
	 * @author Damian Connolly
	 */
	public class Fish extends Sprite implements IUpdateObj
	{
		
		/********************************************************************************/
		
		private static const M_RADIUS_NORMAL:Number = 15.0; // the radius for the fish when at scale 1
		private static const M_COLOUR_TYPE_0:Colour	= Colour.GREEN;
		private static const M_COLOUR_TYPE_1:Colour = Colour.YELLOW;
		private static const M_COLOUR_TYPE_2:Colour = Colour.BLUE;
		private static const M_COLOUR_TYPE_3:Colour = Colour.ORANGE;
		
		/********************************************************************************/
		
		private static var m_helperPoint:Point = new Point;
		
		/********************************************************************************/
		
		/**
		 * Gets the colour for a fish based on it's type
		 * @param type The type of fish
		 * @return The colour for the fish
		 */
		public static function getColourForType( type:int ):Colour
		{
			if ( type == 1 )
				return Fish.M_COLOUR_TYPE_1;
			else if ( type == 2 )
				return Fish.M_COLOUR_TYPE_2;
			else if ( type == 3 )
				return Fish.M_COLOUR_TYPE_3;
			return Fish.M_COLOUR_TYPE_0;
		}
		
		/********************************************************************************/
		
		/**
		 * The signal dispatched when we start going out of the screen. It should take one
		 * parameter of type Fish
		 */
		public var signalOnHitStageLimit:Signal = null;
		
		/**
		 * The signal dispatched when we're destroyed. It should take one parameter of type Fish
		 */
		public var signalOnDestroy:Signal = null;
		
		/**
		 * The radius for the collision detection
		 */
		public var radius:Number = 0.0;
		
		/**
		 * The type of the fish
		 */
		public var type:int = 0;
		
		/**
		 * The scale of the fish, recovered from unity
		 */
		public var scale:Number = 0.0;
		
		/**
		 * Has this fish had a baby?
		 */
		public var hasHadBaby:Boolean = false;
		
		/**
		 * Is the fish dead? I.e. hit the boat
		 */
		public var isDead:Boolean = false;
		
		/**
		 * Is this a baby fish?
		 */
		public var isBaby:Boolean = false;
		
		/**
		 * Have we shown the splash for coming out of the water
		 */
		public var hasShownUpSplash:Boolean = false;
		
		/**
		 * Have we shown the splash for landing in the water
		 */
		public var hasShownDownSplash:Boolean = false;
		
		/********************************************************************************/
		
		private var m_xVel:Number = 0.0;
		private var m_yVel:Number = 0.0; // the velocity for the y
		
		/********************************************************************************/
		
		public function get active():Boolean { return !this.isOutOfScreen || this.m_yVel < 0.0; }
		
		public function get isOutOfScreen():Boolean { return this.y >= DS.stage.stageHeight + this.radius * 0.5; }
		
		public function get isUnderOceanFloor():Boolean { return this.y >= DS.stage.stageHeight - SizeConstants.OCEAN_SIZE_PX; }
		
		public function get speed():Number
		{
			Fish.m_helperPoint.setTo( this.m_xVel, this.m_yVel );
			return Fish.m_helperPoint.length;
		}
		
		public function get isGoingDown():Boolean { return this.m_yVel > 0; }
		
		/********************************************************************************/
		
		public function Fish( scale:Number, type:int = -1 ) 
		{
			// create our type if needed
			if ( type < 0 )
				type = int( Math.random() * 4 );
				
			// add our graphics
			var bm:Bitmap 	= new Bitmap( DS.getAsset( Resources.IMAGE_FISH ).bitmapData );
			bm.x			= -bm.width * 0.5;
			bm.y			= -bm.height * 0.5;
			this.addChild( bm );
			DS.addToUpdate( this );
			
			// get our colour tint
			this.type						= type;
			var c:Colour					= Fish.getColourForType( this.type );
			this.transform.colorTransform 	= c.getColorTransform( 0.7 );
			
			// set our scale
			this.scale	= scale;
			this.scaleX = ( ( Fish.M_RADIUS_NORMAL * 2.0 ) / bm.width ) * this.scale;
			this.scaleY = this.scaleX;
			
			// get our radius, for hittesting
			this.radius	= bm.width * 0.5 * this.scaleX;
			
			this.signalOnHitStageLimit 	= new Signal( Fish );
			this.signalOnDestroy		= new Signal( Fish );
		}
		
		public function destroy():void
		{
			this.signalOnDestroy.dispatch( this );
			
			DS.removeFromUpdate( this );
			
			// signals
			this.signalOnHitStageLimit.removeAll();
			this.signalOnDestroy.removeAll();
			
			// grpahics
			this.removeChildren();
			if ( this.parent != null )
				this.parent.removeChild( this );
				
			// clear our params
			this.signalOnHitStageLimit 	= null;
			this.signalOnDestroy		= null;
		}
		
		/**
		 * Jumps a fish
		 * @param xVel The x velocity of the Fish
		 * @param yVel The y velocity of the Fish
		 */
		public function jump( xVel:Number, yVel:Number ):void
		{
			this.m_xVel = xVel;
			this.m_yVel = yVel;
			this.scaleY *= ( this.m_xVel < 0 ) ? -1 : 1;
		}
		
		public function update( dt:Number ):void
		{
			if ( this.isDead )
			{
				this.destroy()
				return;
			}
			
			// update our vel
			var oceanFloor:Number = DS.stage.stageHeight - SizeConstants.OCEAN_SIZE_PX;
			if( this.y <= oceanFloor || this.m_yVel >= 0.0 )
				this.m_yVel += 9.8 * dt;
				
			// move along
			var prevY:Number = this.y;
			this.x += this.m_xVel;
			this.y += this.m_yVel;
			if ( this.y == prevY )
			{
				DS.warn( this, "HACKING POS" );
				this.m_yVel += 9.8 * dt;
				this.y 		+= this.m_yVel;
			}
			
			// clamp the x
			this.x = ( this.x < this.radius * 0.5 ) ? this.radius * 0.5 :
						( this.x > DS.stage.stageWidth - this.radius * 0.5 ) ? DS.stage.stageWidth - this.radius * 0.5 :
							this.x;
			
			// rotate based on our vel
			var rot:Number 	= MathHelper.radiansToDegrees( Math.atan2( this.m_yVel, this.m_xVel ) );
			this.rotation 	= MathHelper.lerp( this.rotation, rot, 0.1 + ( this.y / DS.stage.stageHeight ) * 0.1 );
			
			// check if we're just going out of screen
			if ( prevY + this.height * 0.5 < DS.stage.stageHeight && 
					this.y + this.height * 0.5 >= DS.stage.stageHeight &&
					this.m_yVel > 0.0 )
			{
				this.rotation = rot;
				this.signalOnHitStageLimit.dispatch( this );
			}
			
			// if we're out of frame, then destroy
			if ( this.isOutOfScreen && this.m_yVel >= 0.0 )
				this.destroy();
		}
		
	}

}