package fishing.logic 
{
	import com.divillysausages.ds.core.IUpdateObj;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.input.Key;
	import com.divillysausages.ds.math.MathHelper;
	import fishing.Main;
	import fishing.state.constants.SizeConstants;
	import fishing.state.constants.StateNames;
	import fishing.state.StatePlay;
	import flash.display.Sprite;
	import flash.geom.Point;
	
	/**
	 * The ship that the player controls
	 * @author Damian Connolly
	 */
	public class Ship extends Sprite implements IUpdateObj
	{
		
		/********************************************************************************/
		
		/**
		 * The speed of the ship
		 */
		public var shipSpeed:Number = 160.0;
		
		/**
		 * The speed of the line
		 */
		//public var lineSpeed:Number = 80.0;
		
		/**
		 * The top of our line
		 */
		//public var lineOffset:Point = new Point;
		
		/**
		 * The bottom of our line
		 */
		//public var lineBottom:Point = new Point;
		
		/********************************************************************************/
		
		private var m_angle:Number 				= 0.0;	// the angle used for our floating anim
		//private var m_line:Shape				= null;	// the line graphics
		private var m_width:Number				= 0.0;	// the ship width
		private var m_height:Number				= 0.0;	// the ship height
		//private var m_lineLen:Number			= 0.0;	// the length of our line
		//private var m_lineMinLen:Number			= 0.0;	// the min length our line can be
		//private var m_lineMaxLen:Number			= 0.0;	// the max length our line can be
		//private var m_lineDesiredBottom:Point	= null;	// the direction for the line
		private var m_prevShipDir:int			= 0;	// the previous ship dir, for when we start/stop
		//private var m_prevLineDir:int			= 0;	// the previous line dir, for when we start/stop
		private var m_seaLevel:Number			= 0.0;	// the sea level on the flash side
		private var m_barrel:Barrel				= null;	// the barrel that we're throwing out
		private var m_rayPoint:Point			= new Point;
		
		/********************************************************************************/
		
		public function get active():Boolean { return true; }
		
		public function get hullWidth():Number { return this.m_width; }
		
		public function get hullHeight():Number { return this.m_height; }
		
		/********************************************************************************/
		
		public function Ship() 
		{
			// get our sea level
			this.m_seaLevel = DS.stage.stageHeight - SizeConstants.OCEAN_SIZE_PX;
			
			// draw our graphics
			var scale:Number	= 1.5;
			this.m_width		= SizeConstants.SHIP_W_M * SizeConstants.SCALE * scale;
			this.m_height		= 20.0 * scale;
			var graphics:boat	= new boat;
			graphics.scaleX		= scale;
			graphics.scaleY		= scale;
			this.addChild( graphics );
			//this.graphics.beginFill( 0xff0000 );
			//this.graphics.drawRect( -this.m_width * 0.5, -this.m_height, this.m_width, this.m_height );
			//this.graphics.endFill();
			this.x			= DS.stage.stageWidth * 0.5;
			this.y			= this.m_seaLevel;
			
			// set the line position
			//this.m_lineDesiredBottom	= new Point;
			//this.lineOffset				= new Point( -this.m_width * 0.5 - 20, -this.m_height * 3 );
			//this.lineBottom				= new Point;
			//this.lineBottom.setTo( this.x + this.lineOffset.x, this.y + this.lineOffset.y );
			
			// calculate our max line length
			//this.m_lineMinLen	= SizeConstants.OCEAN_SIZE_PX;
			//this.m_lineMaxLen 	= SizeConstants.OCEAN_SIZE_PX + Math.abs( this.lineOffset.y ) + SizeConstants.OCEAN_DEPTH_M * SizeConstants.SCALE;
			//this.m_lineLen		= this.m_lineMinLen;
			
			// create our line graphics
			//this.m_line = new Shape;
			//this.addChild( this.m_line );
			
			// tell unity about it immediately
			( DS.main as Main ).js.notifyShipStopMove( this.x );
			//( DS.main as Main ).js.notifyShipPosChange( this.x );
			
			// bind our throw key
			Input.bind( Key.SPACE, this.throwBarrel, Input.KEY_UP );
			
			// add us to the update
			DS.addToUpdate( this );
		}
		
		public function throwBarrel():void
		{
			var main:Main = ( DS.main as Main );
			
			// if we have a barrel, explode it if it's out of the screen
			if ( this.m_barrel != null )
			{				
				// check if it's out of the screen
				if ( this.m_barrel.isOutOfScreen )
				{
					main.js.notifyLoveTNTExplode();
					this.m_barrel = null;
				}
			}
			else
			{				
				// get our position of where we're going to throw it
				var bx:Number = this.x - ( this.m_width * 0.5 ) * this.scaleX;
				var by:Number = this.y - this.m_height;
				
				// create a barrel and throw it
				this.m_barrel = new Barrel;
				this.m_barrel.signalOnHitStageLimit.addOnce( this._onBarrelHitStageLimit );
				this.m_barrel.x	= bx;
				this.m_barrel.y = by - this.m_barrel.height * 0.5;
				this.m_barrel.throwOut( -this.scaleX * 2, -6 );
				main.addChildAt( this.m_barrel, StatePlay.shipDepth );
			}
		}
		
		/**
		 * Called every frame the ship is active
		 * @param dt The delta time since the last update
		 */
		public function update( dt:Number ):void
		{
			// move our y
			this.m_angle 	+= dt;// * Math.PI;
			this.y			= this.m_seaLevel + Math.sin( this.m_angle ) * 5;
			
			// move our x
			var main:Main = ( DS.main as Main );
			
			// if no controls are down, do nothing
			if ( main.controls.isLeftPressed || main.controls.isRightPressed )
			{	
				// move along
				var lb:Number	= this.width * 0.5;							// left bounds
				var rb:Number	= DS.stage.stageWidth - this.width * 0.5; 	// right bounds
				var oldX:Number	= this.x;
				this.x 			+= ( main.controls.isLeftPressed ) ? -this.shipSpeed * dt : this.shipSpeed * dt;
				this.x 			= ( this.x < lb ) ? lb : ( this.x > rb ) ? rb : this.x;
				
				var xScale:Number = ( main.controls.isLeftPressed ) ? -1 : 1;
				if ( this.scaleX != xScale )
					this.scaleX = xScale;
					
				this.rotation = MathHelper.lerp( this.rotation, ( main.controls.isLeftPressed ) ? 15 : -15, 0.1 );
				
				// if our line was moving, tell unity that it stopped
				//if ( this.m_prevLineDir != 0 )
				//{
					//main.js.notifyLineStopMove( this.m_lineLen );
					//this.m_prevLineDir = 0;
				//}
									
				// tell js so it can tell unity
				//main.js.notifyShipPosChange( this.x );
				var dir:int = ( main.controls.isLeftPressed ) ? -1 : 1;
				if ( this.m_prevShipDir != dir )
					main.js.notifyShipStartMove( dir );
				this.m_prevShipDir = dir;
			}
			else if ( this.m_prevShipDir != 0 )
			{
				main.js.notifyShipStopMove( this.x );
				this.m_prevShipDir = 0;
			}
			/*else if ( main.controls.isUpPressed || main.controls.isDownPressed )
			{
				// extend/retract our line
				this.m_lineLen += ( main.controls.isUpPressed ) ? -this.lineSpeed * dt : this.lineSpeed * dt;
				this.m_lineLen	= ( this.m_lineLen < this.m_lineMinLen ) ? this.m_lineMinLen : 
									( this.m_lineLen > this.m_lineMaxLen ) ? this.m_lineMaxLen : 
										this.m_lineLen;
											
				// check if we've started moving
				dir = ( main.controls.isUpPressed ) ? -1 : 1;
				if ( this.m_prevLineDir != dir )
					main.js.notifyLineStartMove( dir );
				this.m_prevLineDir = dir;
			}
			else if ( this.m_prevLineDir != 0 )
			{
				main.js.notifyLineStopMove( this.m_lineLen );
				this.m_prevLineDir = 0;
			}*/
			
			// return the rotation if we're not moving
			if ( !main.controls.isLeftPressed && !main.controls.isRightPressed )
				this.rotation = MathHelper.lerp( this.rotation, 0, 0.1 );
			
			// draw our line support
			//this.m_line.graphics.clear();
			//this.m_line.graphics.beginFill( 0 );
			//this.m_line.graphics.moveTo( -this.m_width * 0.5 + 10, -this.m_height );
			//this.m_line.graphics.lineTo( -this.m_width * 0.5 + 5, -this.m_height * 1.5 );
			//this.m_line.graphics.lineTo( this.lineOffset.x, this.lineOffset.y );
			//this.m_line.graphics.lineTo( -this.m_width * 0.5 + 10, -this.m_height * 2.0 );
			//this.m_line.graphics.lineTo( -this.m_width * 0.5 + 30, -this.m_height );
			//this.m_line.graphics.moveTo( -this.m_width * 0.5 + 10, -this.m_height );
				
			// get the line desired bottom
			//this.m_lineDesiredBottom.setTo( this.x + this.lineOffset.x, this.y + this.lineOffset.y + this.m_lineLen );
			
			// lerp the line bottom x
			//this.lineBottom.x = MathHelper.lerp( this.lineBottom.x, this.m_lineDesiredBottom.x, 1.0 * dt );
			//this.lineBottom.y = MathHelper.lerp( this.lineBottom.y, this.m_lineDesiredBottom.y, 1.0 * dt );
			
			// draw our line
			//this.m_line.graphics.lineStyle( 2.0, 0 );
			//this.m_line.graphics.moveTo( this.lineOffset.x, this.lineOffset.y );
			//this.m_line.graphics.lineTo( this.lineBottom.x - this.x, this.lineBottom.y - this.y );
			
			// tell js, so it can tell unity
			//main.js.notifyLinePosChange( this.x + this.lineTop.x, this.y + this.lineTop.y,
										//this.x + this.lineBottom.x, this.y + this.lineBottom.y );
										
			// show our splash if we need to
			if ( this.m_barrel != null )
			{				
				// NOTE: pre-empt it as it takes a minute to arrive
				if ( !this.m_barrel.hasShownSplash && this.m_barrel.isGoingDown && this.m_barrel.y + this.m_barrel.origHeight + 30 >= this.m_seaLevel )
				{
					( ( DS.main as Main ).fsm.getState( StateNames.PLAY ) as StatePlay ).showSplash( this.m_barrel.x + (( this.m_barrel.isGoingRight ) ? 20 : -20 ), this.m_seaLevel );
					this.m_barrel.hasShownSplash = true;
				}
			}
		}
		
		/********************************************************************************/
		
		// called when our barrel hits the stage limit - tell js
		private function _onBarrelHitStageLimit():void
		{
			( DS.main as Main ).js.notifyLoveTNT( this.m_barrel.x );
		}
		
	}

}