package fishing.network 
{
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.input.Input;
	import com.divillysausages.ds.input.Key;
	import com.divillysausages.ds.util.EIUtils;
	import fishing.logic.Fish;
	import fishing.state.constants.SizeConstants;
	import org.osflash.signals.Signal;
	
	/**
	 * The class that will deal with our JS calls
	 * @author Damian Connolly
	 */
	public class Ext 
	{
		
		/********************************************************************************/
		
		//private static const M_CALL_ON_KEY_DOWN:String		= "Fish.OnKeyDown";		// the function to call when we get a key press
		//private static const M_CALL_ON_KEY_UP:String			= "Fish.OnKeyUp";		// the function to call when we get a key release
		private static const M_CALL_READY:String 				= "Fish.FlashReady";	// the function to call to tell js we're ready
		private static const M_CALL_ON_PAUSE_CHANGE:String		= "Fish.OnPauseChange";	// the function to call when the pause changes
		private static const M_CALL_ON_SHIP_START_MOVE:String	= "Fish.OnShipStartMove";	// the function to call when we start to move
		private static const M_CALL_ON_SHIP_STOP_MOVE:String	= "Fish.OnShipStopMove";	// the function to call when we've stopped moving
		private static const M_CALL_ON_LOVE_TNT:String			= "Fish.OnLoveTNT";			// the function to call when a tnt barrel is off screen
		private static const M_CALL_ON_LOVE_TNT_EXPLODE:String	= "Fish.OnLoveTNTExplode";	// the function to call when we want to blow up the barrel
		private static const M_CALL_ON_FISH_LAND:String			= "Fish.OnFishLand";		// the function to call when a fish has landed
		//private static const M_CALL_ON_LINE_START_MOVE:String	= "Fish.OnLineStartMove";	// the function to call when we start moving the line
		//private static const M_CALL_ON_LINE_STOP_MOVE:String	= "Fish.OnLineStopMove";	// the function to call when we stop moving the line
		//private static const M_CALL_SHIP_POS_CHANGE:String	= "Fish.ShipPosChange";	// the function to call to tell js that the ship's position has changed
		//private static const M_CALL_LINE_POS_CHANGE:String	= "Fish.LinePosChange";	// the function to call to tell js that the line's position has changed
		
		//private static const M_LISTEN_ON_KEY_DOWN:String		= "OnKeyDown";	// the function called when we get a key down
		//private static const M_LISTEN_ON_KEY_UP:String			= "OnKeyUp";	// the function called when we get a key up
		private static const M_LISTEN_READY:String 				= "GlobalReady";// the function called when both plugins are ready
		private static const M_LISTEN_ON_FISH_JUMP:String		= "OnFishJump";	// the function called when we get a jumping fish
		
		/********************************************************************************/
		
		/**
		 * The signal dispatched when we're ready to move (flash and unity). It should
		 * take no parameters
		 */
		public var signalOnGlobalReady:Signal = null;
		
		/**
		 * The signal dispatched when a fish has jumped, It should take one parameter
		 * of type Number (x), one of type Number (angle), one of type int (type), one
		 * of type Number (scale), and one of type Number (speed)
		 */
		public var signalOnFishJump:Signal = null;
		
		/**
		 * The signal dispatched when we get a key down from JS. It should take one parameter
		 * of type Key
		 */
		//public var signalOnKeyDown:Signal = null;
		
		/**
		 * The signal dispatched when we get a key down from JS. It should take one parameter
		 * of type Key
		 */
		//public var signalOnKeyUp:Signal = null;
		
		/********************************************************************************/
		
		public function Ext() 
		{
			// listen for our ready
			//EIUtils.addCallback( Ext.M_LISTEN_ON_KEY_DOWN, this._onListenKeyDown );
			//EIUtils.addCallback( Ext.M_LISTEN_ON_KEY_UP, this._onListenKeyUp );
			EIUtils.addCallback( Ext.M_LISTEN_READY, this._onListenReady );
			EIUtils.addCallback( Ext.M_LISTEN_ON_FISH_JUMP, this._onListenFishJump );
			//EIUtils.addCallback( Ext.M_LISTEN_SHIP_POS_CHANGE, this._onListenShipPosChange );
			
			// create our signals
			this.signalOnGlobalReady 	= new Signal;
			this.signalOnFishJump		= new Signal( Number, Number, int, Number, Number ); // pos, angle, type, scale, speed
			//this.signalOnKeyDown		= new Signal( Key );
			//this.signalOnKeyUp			= new Signal( Key );
		}
		
		/**
		 * Notifies JS of a key down
		 * @param keyCode The key code of the key pressed
		 */
		/*public function notifyKeyDown( keyCode:int ):void
		{
			// local
			if ( !EIUtils.isAvailable )
			{
				this._onListenKeyDown( keyCode );
				return;
			}
			EIUtils.call( Ext.M_CALL_ON_KEY_DOWN, keyCode );
		}*/
		
		/**
		 * Notifies JS of a key released
		 * @param keyCode The key code of the key released
		 */
		/*public function notifyKeyUp( keyCode:int ):void
		{
			// local
			if ( !EIUtils.isAvailable )
			{
				this._onListenKeyUp( keyCode );
				return;
			}
			EIUtils.call( Ext.M_CALL_ON_KEY_UP, keyCode );
		}*/
		
		/**
		 * Notifies JS that we're ready to go
		 */
		public function notifyReady():void
		{
			// local
			if ( !EIUtils.isAvailable )
			{
				this._onListenReady();
				return;
			}
			EIUtils.call( Ext.M_CALL_READY );
		}
		
		/**
		 * Notifies JS that the pause value has changed
		 * @param isPaused Is the game paused?
		 */
		public function notifyPauseChanged( isPaused:Boolean ):void
		{
			EIUtils.call( Ext.M_CALL_ON_PAUSE_CHANGE, isPaused );
		}
		
		/**
		 * Notifies JS that we've started moving
		 * @param dir The direction that we're moving in
		 */
		public function notifyShipStartMove( dir:int ):void
		{
			EIUtils.call( Ext.M_CALL_ON_SHIP_START_MOVE, dir );
		}
		
		/**
		 * Notifies JS that we've stopped moving
		 * @param x The ship's x position
		 */
		public function notifyShipStopMove( x:Number ):void
		{
			var xPC:Number 	= ( x / DS.stage.stageWidth );
			xPC				= ( xPC < 0.0 ) ? 0.0 : ( xPC > 1.0 ) ? 1.0 : xPC;
			EIUtils.call( Ext.M_CALL_ON_SHIP_STOP_MOVE, xPC );
		}
		
		/**
		 * Notifies JS that we've thrown out a love barrel
		 * @param x The barrel's x position
		 */
		public function notifyLoveTNT( x:Number ):void
		{
			var xPC:Number 	= ( x / DS.stage.stageWidth );
			xPC				= ( xPC < 0.0 ) ? 0.0 : ( xPC > 1.0 ) ? 1.0 : xPC;
			EIUtils.call( Ext.M_CALL_ON_LOVE_TNT, xPC );
		}
		
		/**
		 * Notifies JS that we want to explode our barrel
		 */
		public function notifyLoveTNTExplode():void
		{
			EIUtils.call( Ext.M_CALL_ON_LOVE_TNT_EXPLODE );
		}
		
		/**
		 * Notifies JS that a fish has landed
		 * @param fish The fish that landed
		 */
		public function notifyFishLand( fish:Fish ):void
		{
			// get the position
			var xPC:Number 	= ( fish.x / DS.stage.stageWidth );
			xPC				= ( xPC < 0.0 ) ? 0.0 : ( xPC > 1.0 ) ? 1.0 : xPC;
			
			// get the angle
			trace( "Angle: " + fish.rotation );
			var angle:Number 	= fish.rotation;
			var type:int		= fish.type;
			var scale:Number	= fish.scale;
			var speed:Number	= fish.speed;
			EIUtils.call( Ext.M_CALL_ON_FISH_LAND, xPC.toFixed( 2 ), angle.toFixed( 2 ), type, scale.toFixed( 2 ), speed.toFixed( 2 ) );
		}
		
		/**
		 * Called when the line has started to move
		 * @param dir The direction that the line is moving in
		 */
		/*public function notifyLineStartMove( dir:int ):void
		{
			EIUtils.call( Ext.M_CALL_ON_LINE_START_MOVE, dir );
		}*/
		
		/**
		 * Called when our line has stopped moving
		 * @param y The line y position
		 */
		/*public function notifyLineStopMove( y:Number ):void
		{
			// call it (convert y to meters)
			EIUtils.call( Ext.M_CALL_ON_LINE_STOP_MOVE, y * SizeConstants.INV_SCALE );
		}*/
		
		/**
		 * Notifies JS that the position of our ship has changed
		 * @param x The new x position of the ship
		 */
		/*public function notifyShipPosChange( x:Number ):void
		{
			var xPC:Number 	= ( x / DS.stage.stageWidth );
			xPC				= ( xPC < 0.0 ) ? 0.0 : ( xPC > 1.0 ) ? 1.0 : xPC;
			EIUtils.call( Ext.M_CALL_SHIP_POS_CHANGE, xPC );
		}*/
		
		/**
		 * Notifies JS that the position of our line has changed
		 * @param sx The start x of the line
		 * @param sy The start y of the line
		 * @param ex The end x of the line
		 * @param ey The end y of the line
		 */
		/*public function notifyLinePosChange( sx:Number, sy:Number, ex:Number, ey:Number ):void
		{
			// remove the height of the swf from the y
			sy -= DS.stage.stageHeight;
			ey -= DS.stage.stageHeight;
			
			// convert the y to meters
			var scale:Number = SizeConstants.INV_SCALE;
			sy 				*= scale;
			ey 				*= scale;
			
			// convert the x between 0 and 1
			var sxPC:Number 	= ( sx / DS.stage.stageWidth );
			var exPC:Number 	= ( ex / DS.stage.stageWidth );
			sxPC				= ( sxPC < 0.0 ) ? 0.0 : ( sxPC > 1.0 ) ? 1.0 : sxPC;
			exPC				= ( exPC < 0.0 ) ? 0.0 : ( exPC > 1.0 ) ? 1.0 : exPC;
			
			// call it
			EIUtils.call( Ext.M_CALL_LINE_POS_CHANGE, sxPC, sy, exPC, ey );
		}*/
		
		/********************************************************************************/
		
		// called when we get a keydown press
		/*private function _onListenKeyDown( keyCode:int ):void
		{
			// map unity arrow keys to flash
			if ( keyCode == 273 ) keyCode = 38;	// up
			if ( keyCode == 274 ) keyCode = 40;	// down
			if ( keyCode == 275 ) keyCode = 39;	// right
			if ( keyCode == 276 ) keyCode = 37;	// left
			
			// get our key and dispatch our signal
			//DS.log( this, "Key down " + keyCode );
			var key:Key = Input.getKeyForCode( keyCode );
			if ( key != null )
				this.signalOnKeyDown.dispatch( key );
		}*/
		
		// called when we get a keyup press
		/*private function _onListenKeyUp( keyCode:int ):void
		{
			// map unity arrow keys to flash
			if ( keyCode == 273 ) keyCode = 38;	// up
			if ( keyCode == 274 ) keyCode = 40;	// down
			if ( keyCode == 275 ) keyCode = 39;	// right
			if ( keyCode == 276 ) keyCode = 37;	// left
			
			// get our key and dispatch our signal
			//DS.log( this, "Key up " + keyCode );
			var key:Key = Input.getKeyForCode( keyCode );
			if ( key != null )
				this.signalOnKeyUp.dispatch( key );
		}*/
		
		// callback for when we're ready
		private function _onListenReady():void
		{
			DS.log( this, "Global ready received" );
			EIUtils.removeCallback( Ext.M_LISTEN_READY );
			this.signalOnGlobalReady.dispatch();
		}
		
		// called when a fish has jumped
		private function _onListenFishJump( params:String ):void
		{
			var paramAr:Array 	= params.split( "," );
			var xPC:Number		= Number( paramAr[0] );
			var angle:Number	= Number( paramAr[1] );
			var type:int 		= int( paramAr[2] );
			var scale:Number	= Number( paramAr[3] );
			var speed:Number	= Number( paramAr[4] );
			
			// convert the x
			var x:Number = DS.stage.stageWidth * xPC;
			
			// conver the angle
			angle = -angle;
			
			// convert the speed
			speed *= SizeConstants.INV_SCALE * 3;
			speed = ( speed < 5 ) ? 5 : ( speed > 12 ) ? 12 : speed;
			
			// dispatch our signal
			this.signalOnFishJump.dispatch( x, angle, type, scale, speed );
		}
		
	}

}