package fishing.state 
{
	import com.divillysausages.ds.core.GameState;
	import com.divillysausages.ds.DS;
	import com.divillysausages.ds.math.MathHelper;
	import com.divillysausages.ds.particles.Emitter;
	import com.divillysausages.ds.particles.IParticle;
	import fishing.assets.Resources;
	import fishing.logic.Fish;
	import fishing.logic.Ship;
	import fishing.logic.Splash;
	import fishing.logic.Wave;
	import fishing.Main;
	import fishing.particle.ImageParticle;
	import fishing.state.constants.SizeConstants;
	import fishing.state.constants.StateNames;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.text.TextFormat;
	
	/**
	 * The state where we actually play
	 * @author Damian Connolly
	 */
	public class StatePlay extends GameState
	{
		
		/********************************************************************************/
		
		private static var m_ship:Ship = null; // the ship, so we can get the depth
		
		/********************************************************************************/
		
		/**
		 * The ship depth, so we can add stuff between the waves
		 */
		public static function get shipDepth():int
		{
			return StatePlay.m_ship.parent.getChildIndex( StatePlay.m_ship );
		}
		
		/********************************************************************************/
		
		/**
		 * The ship that we're controlling
		 */
		public var ship:Ship = null;
		
		/**
		 * The vector of fish that we have
		 */
		public var fishes:Vector.<Fish> = new Vector.<Fish>;
		
		/********************************************************************************/
		
		private var m_helperPoint:Point 				= new Point;
		private var m_txtScore:TextField				= null;
		private var m_numBabies:int						= 0;
		private var m_emittersHearts:Vector.<Emitter>	= null; // the emitters that we're using
		private var m_emittersSkulls:Vector.<Emitter>	= null; // the emitters for when fish land in the boat
		private var m_boatCollisionRect:Rectangle		= null;
		private var m_seaLevel:Number					= 0.0;
		private var m_pacifistMode:Boolean				= false;// do fish die when they land on your ship?
		private var m_btnPacifist:Sprite				= null;	// the button to unpause us
		
		/********************************************************************************/
		
		public function StatePlay() 
		{
			super( StateNames.PLAY );
			this.m_emittersHearts 	= new Vector.<Emitter>;
			this.m_emittersSkulls 	= new Vector.<Emitter>;
			this.m_seaLevel			= DS.stage.stageHeight - SizeConstants.OCEAN_SIZE_PX;
		}
		
		override public function begin():void 
		{
			var main:Main = ( DS.main as Main );
			
			var w:Wave = new Wave( -40 );
			main.addChild( w );
			
			w = new Wave( -20 );
			main.addChild( w );
			
			// create our ship
			this.ship 			= new Ship;
			StatePlay.m_ship	= this.ship;
			main.addChild( this.ship );
			
			w = new Wave( -2 );
			main.addChild( w );
			
			w = new Wave( 15 );
			main.addChild( w );
			
			// listen for our fish jump event
			main.js.signalOnFishJump.add( this._onFishJump );
			
			// create our score textfield
			var tf:TextFormat 					= new TextFormat( "Trebuchet Ms", 14, 0xffffff, true );
			this.m_txtScore						= new TextField;
			this.m_txtScore.defaultTextFormat	= tf;
			this.m_txtScore.autoSize			= TextFieldAutoSize.LEFT;
			this.m_txtScore.selectable			= false;
			this.m_txtScore.mouseEnabled		= false;
			this.m_txtScore.mouseWheelEnabled	= false;
			this.m_txtScore.wordWrap			= false;
			this.m_txtScore.multiline			= false;
			this.m_txtScore.text				= "# Fish babies: 0";
			this.m_txtScore.x					= 10;
			this.m_txtScore.y					= DS.stage.stageHeight - this.m_txtScore.height - 10;
			DS.stage.addChild( this.m_txtScore );
			
			this.m_boatCollisionRect = new Rectangle;
			
			// text
			tf 					= new TextFormat( "Trebuchet Ms", 12, 0xffffff, true );
			var t:TextField		= new TextField;
			t.defaultTextFormat	= tf;
			t.autoSize			= TextFieldAutoSize.LEFT;
			t.selectable		= false;
			t.mouseEnabled		= false;
			t.mouseWheelEnabled	= false;
			t.wordWrap			= false;
			t.multiline			= false;
			t.text				= ( this.m_pacifistMode ) ? "Pacifist: ON" : "Pacifist: OFF";
			
			// create our pacifist button
			this.m_btnPacifist = new Sprite;
			this.m_btnPacifist.graphics.lineStyle( 2.0, 0xffffff );
			this.m_btnPacifist.graphics.beginFill( 0x000066 );
			this.m_btnPacifist.graphics.drawRect( 0.0, 0.0, 85.0, 25.0 );
			this.m_btnPacifist.graphics.endFill();
			this.m_btnPacifist.x = 10;
			this.m_btnPacifist.y = 10;
			this.m_btnPacifist.addEventListener( MouseEvent.CLICK, function( me:MouseEvent ):void
			{
				m_pacifistMode 	= !m_pacifistMode;
				t.text			= ( m_pacifistMode ) ? "Pacifist: ON" : "Pacifist: OFF";
			});
			
			// add the text
			t.x	= this.m_btnPacifist.width * 0.5 - t.width * 0.5;
			t.y	= this.m_btnPacifist.height * 0.5 - t.height * 0.5;
			this.m_btnPacifist.addChild( t );
			main.addChild( this.m_btnPacifist );
		}
		
		// shows a splash at a specific point
		public function showSplash( x:Number, y:Number ):void
		{
			var splash:Splash = new Splash;
			splash.show( x, y );
			( DS.main as Main ).addChildAt( splash, StatePlay.shipDepth + 1 );
		}
		
		override public function update(dt:Number):void 
		{
			this.m_boatCollisionRect.x 		= this.ship.x - this.ship.hullWidth * 0.5;
			this.m_boatCollisionRect.y 		= this.ship.y - this.ship.hullHeight - 15;
			this.m_boatCollisionRect.width	= this.ship.hullWidth;
			this.m_boatCollisionRect.height	= this.ship.hullHeight + 15;
			
			// check our fish collisions
			var len:int = this.fishes.length;
			outer: for ( var i:int = 0; i < len; i++ )
			{
				var fishA:Fish = this.fishes[i];
				if ( fishA.isDead )
					continue;
				
				// check if the fish hit the boat
				if ( fishA.isGoingDown && !this.m_pacifistMode )
				{
					if ( this.m_boatCollisionRect.contains( fishA.x, fishA.y ) )
					{
						fishA.isDead 	= true;
						fishA.visible	= false;
						this._emitSkulls( fishA.x, fishA.y );
						continue;
					}
					else if ( !fishA.hasShownDownSplash && ( fishA.y + 10 ) >= this.m_seaLevel )
					{
						this.showSplash( fishA.x, this.m_seaLevel );
						fishA.hasShownDownSplash = true;
					}
				}
				//else
				//{
					//if ( !fishA.hasShownUpSplash && !fishA.isBaby && fishA.y <= DS.stage.stageHeight )
					//{
						//this.showSplash( fishA.x, this.m_seaLevel );
						//fishA.hasShownUpSplash = true;
					//}
				//}
				
				// if it's had a baby, or is under the ocean floor, we don't care
				if ( fishA.hasHadBaby || fishA.isUnderOceanFloor )
					continue;
					
				for ( var j:int = 0; j < len; j++ )
				{
					var fishB:Fish = this.fishes[j];
					if ( fishA == fishB || fishB.hasHadBaby || fishB.isUnderOceanFloor )
						continue;
						
					// compare them
					var dx:Number 	= fishA.x - fishB.x;
					var dy:Number 	= fishA.y - fishB.y;
					var dist:Number = ( dx * dx ) + ( dy * dy );
					var r2:Number	= ( fishA.radius + fishB.radius ) * ( fishA.radius + fishB.radius );
					if ( dist < r2 )
					{
						// new baby!
						fishA.hasHadBaby = true;
						fishB.hasHadBaby = true;
						
						// spawn a new fish
						var baby:Fish 	= new Fish( ( fishA.scale + fishB.scale ) * 0.5, ( Math.random() < 0.5 ) ? fishA.type : fishB.type );
						baby.x			= ( fishA.x + fishB.x ) * 0.5;
						baby.y			= ( fishA.y + fishB.y ) * 0.5;
						baby.isBaby		= true;
						baby.signalOnHitStageLimit.add( this._onFishHitStageLimit );
						baby.signalOnDestroy.add( this._onFishDestroy );
						
						// set our initial rotation
						baby.rotation = -90;
						baby.jump( 0.0, -5.0 - Math.random() * 5.0 );
						( DS.main as Main ).addChildAt( baby, StatePlay.shipDepth + 1 );
						
						// add it to our vector
						this.fishes.push( baby );
						
						// update our score
						this.m_numBabies++;
						this.m_txtScore.text = "# Fish babies: " + this.m_numBabies;
						
						// create a new emitter
						this._emitHearts( baby.x, baby.y );						
						continue outer;
					}
				}
			}
			
			// update our emitters
			for each( var emitter:Emitter in this.m_emittersHearts )
			{
				// if the emitter's not emitting, do nothing
				if ( !emitter.active )
					continue;
					
				// update the particles
				var particles:Vector.<IParticle> = emitter.particles;
				for each( var ip:ImageParticle in particles )
				{
					if ( ip.visible && ip.parent == null )
						DS.canvas.addChild( ip );
					else if ( !ip.visible && ip.parent != null )
						ip.parent.removeChild( ip );
				}
			}
			for each( emitter in this.m_emittersSkulls )
			{
				// if the emitter's not emitting, do nothing
				if ( !emitter.active )
					continue;
					
				// update the particles
				particles = emitter.particles;
				for each( ip in particles )
				{
					if ( ip.visible && ip.parent == null )
						DS.canvas.addChild( ip );
					else if ( !ip.visible && ip.parent != null )
						ip.parent.removeChild( ip );
				}
			}
		}
		
		/********************************************************************************/
		
		// called when we get a fish jumping
		private function _onFishJump( x:Number, angle:Number, type:int, scale:Number, speed:Number ):void
		{
			// create our fish
			var fish:Fish 	= new Fish( scale, type );
			fish.x			= x;
			fish.y			= DS.stage.stageHeight;
			fish.signalOnHitStageLimit.add( this._onFishHitStageLimit );
			fish.signalOnDestroy.add( this._onFishDestroy );
			
			// get our velocity
			this.m_helperPoint.setTo( 1.0, 0.0 );
			MathHelper.rotate( this.m_helperPoint, MathHelper.degreesToRadians( angle ), this.m_helperPoint );
			this.m_helperPoint.normalize( speed );
			
			// set our initial rotation
			fish.rotation = MathHelper.degreeAngle( this.m_helperPoint );
			fish.jump( this.m_helperPoint.x, this.m_helperPoint.y );
			( DS.main as Main ).addChildAt( fish, StatePlay.shipDepth + 1 );
			
			// add it to our vector
			this.fishes.push( fish );
		}
		
		// called when a fish is being destroyed
		private function _onFishDestroy( fish:Fish ):void
		{
			// remove it from our vector
			var index:int = this.fishes.indexOf( fish );
			if ( index != -1 )
				this.fishes.splice( index, 1 );
		}
		
		// called when our fish has hit the stage limit
		private function _onFishHitStageLimit( fish:Fish ):void
		{
			// notify js
			( DS.main as Main ).js.notifyFishLand( fish );
		}
		
		// emits some hearts
		private function _emitHearts( babyX:Number, babyY:Number ):void
		{
			var emitter:Emitter = this._getEmitter();
			var minRads:Number	= 0.0;
			var maxRads:Number	= Math.PI * 2.0;
			var force:Number	= 60.0;
			emitter.emit( babyX, babyY, minRads, maxRads, force );
			
			// play our sfx
			DS.soundManager.playSound( Resources.SOUND_BABY );
		}
		
		// emits some skulls
		private function _emitSkulls( fishX:Number, fishY:Number ):void
		{
			var emitter:Emitter = this._getEmitter( false );
			var minRads:Number	= 0.0;
			var maxRads:Number	= -Math.PI;
			var force:Number	= 80.0;
			emitter.emit( fishX, fishY, minRads, maxRads, force );
		}
		
		// gets a free emitter, or creates one
		private function _getEmitter( isHearts:Boolean = true ):Emitter
		{
			if ( isHearts )
			{
				for each( var e:Emitter in this.m_emittersHearts )
				{
					if ( !e.active )
						return e;
				}
			}
			else
			{
				for each( e in this.m_emittersSkulls )
				{
					if ( !e.active )
						return e;
				}
			}
			return this._createEmitter( isHearts );
		}
		
		// creates an emitter
		private function _createEmitter( isHearts:Boolean ):Emitter
		{
			var maxParticles:int				= ( isHearts ) ? 40 : 20;
			var life:Number						= ( isHearts ) ? 1.5 : 1.0;
			var particles:Vector.<BitmapData> 	= ( isHearts ) ? Vector.<BitmapData>( [DS.getAsset( Resources.IMAGE_HEART ).bitmapData] ) :
													Vector.<BitmapData>( [DS.getAsset( Resources.IMAGE_SKULL ).bitmapData] );
			var emitter:Emitter 				= new Emitter( ImageParticle, maxParticles, maxParticles / 8, false );
			emitter.initParticles( particles, life );
			
			// add it to our vector and return
			if( isHearts )
				this.m_emittersHearts.push( emitter );
			else
				this.m_emittersSkulls.push( emitter );
			return emitter;
		}
		
	}

}